import convertapi
import os.path

class UploadIO:
    def __init__(self, io, filename = None):
        self.io = io
        self._filename = filename
        self._result = None

    @property
    def file_id(self):
        return self.__result['FileId']

    @property
    def file_ext(self):
        return self.__result['FileExt']

    @property
    def __result(self):
        if self._result is None:
            self._result = convertapi.client.upload(self.io, self.__build_filename())

        return self._result

    def __build_filename(self):
        if self._filename:
            return self._filename

        if 'name' in dir(self.io):
            return os.path.basename(self.io.name)

        raise 'Filename must be provided for non File resources'



import multiprocessing

def map_in_parallel(f, values, pool_size):
    if pool_size < 2:
        return map(f, values)

    pool = multiprocessing.Pool(pool_size)
    results = pool.map_async(f, values)
    pool.close()
    pool.join()

    return results.get()



from os.path import splitext
from io import FileIO
from requests import utils
from .result import Result
from .upload_io import UploadIO

def detect(resource):
    if isinstance(resource, UploadIO):
        return resource.file_ext
    elif 'filename' in dir(resource):
        path = resource.filename
    elif isinstance(resource, FileIO):
        path = resource.name
    elif isinstance(resource, Result):
        path = resource.file.filename
    else:
        path = utils.urlparse(resource).path

    return splitext(path)[1][1:].lower()



import requests
import convertapi

from io import BytesIO
from .exceptions import *

class Client:
	def get(self, path, params = {}, timeout = None):
		timeout = timeout or convertapi.timeout
		r = requests.get(self.url(path), params = params, headers = self.headers(), timeout = timeout)
		return self.handle_response(r)

	def post(self, path, payload, timeout = None):
		timeout = timeout or convertapi.timeout
		r = requests.post(self.url(path), data = payload, headers = self.headers(), timeout = timeout)
		return self.handle_response(r)

	def upload(self, io, filename):
		url = convertapi.base_uri + 'upload'
		encoded_filename = requests.utils.quote(filename)

		headers = self.headers()
		headers.update({
			'Content-Disposition': "attachment; filename*=UTF-8''" + encoded_filename,
		})

		r = requests.post(url, data = io, headers = headers, timeout = convertapi.upload_timeout)
		return self.handle_response(r)

	def download(self, url, path):
		r = requests.get(url, stream = True, timeout = convertapi.download_timeout)

		with open(path, 'wb') as f:
			for chunk in r.iter_content(chunk_size = 1024):
				if chunk:
					f.write(chunk)

		return path

	def download_io(self, url):
		response = requests.get(url, timeout = convertapi.download_timeout)
		return BytesIO(response.content)

	def handle_response(self, r):
		try:
			r.raise_for_status()
		except requests.RequestException as e:
			try:
				raise ApiError(r.json())
			except ValueError:
				raise e

		return r.json()

	def url(self, path):
		return "%s%s?Secret=%s" % (convertapi.base_uri, path, convertapi.api_secret)

	def headers(self):
		return {
			'User-Agent': convertapi.user_agent,
		}



class BaseError(BaseException):
	pass

class ApiError(BaseError):
	def __init__(self, result):
		self.message = result.get('Message', '[message not set]')

		super(ApiError, self).__init__(self.message)

		self.code = result.get('Code', '')
		self.invalid_parameters = result.get('InvalidParameters', '')

	def __str__(self):
		message = "%s Code: %s. %s" % (self.message, self.code, self.invalid_parameters)
		return message.strip()



from .result_file import ResultFile

class Result:
    def __init__(self, response):
        self.response = response

    @property
    def conversion_cost(self):
        return self.response['ConversionCost']

    @property
    def file(self):
        return self.files[0]

    @property
    def files(self):
        return list(map(ResultFile, self.response['Files']))

    def save_files(self, path):
        return [file.save(path) for file in self.files]



import convertapi
import os

class ResultFile:
    def __init__(self, info):
        self.info = info

    @property
    def url(self):
        return self.info['Url']

    @property
    def filename(self):
        return self.info['FileName']

    @property
    def size(self):
        return self.info['FileSize']

    @property
    def io(self):
        if not hasattr(self, '_io'):
            self._io = convertapi.client.download_io(self.url)

        return self._io

    def save(self, path):
        if os.path.isdir(path):
            path = os.path.join(path, self.filename)

        return convertapi.client.download(self.url, path)



import convertapi

from .task import Task

def convert(to_format, params, from_format = None, timeout = None):
    task = Task(from_format, to_format, params, timeout = timeout)
    return task.run()

def user(timeout = None):
    return convertapi.client.get('user', timeout = timeout)



#!/usr/bin/env python

from setuptools.depends import get_module_constant
from setuptools import setup

setup(
    name = 'convertapi',
    packages = ['convertapi'],
    version = get_module_constant('convertapi', '__version__'),
    description = 'Convert API Python Client',
    long_description = 'Convert various files like MS Word, Excel, PowerPoint, Images to PDF and Images. Create PDF and Images from url and raw HTML. Extract and create PowerPoint presentation from PDF. Merge, Encrypt, Split, Repair and Decrypt PDF files. All supported files conversions and manipulations can be found at https://www.convertapi.com/doc/supported-formats',
    author = 'Tomas Rutkauskas',
    author_email = 'support@convertapi.com',
    url = 'https://github.com/ConvertAPI/convertapi-python',
    download_url = 'https://github.com/ConvertAPI/convertapi-python',
    keywords = ['convert', 'api', 'client', 'conversion'],
    install_requires= ['requests>=2.4.2'],
    classifiers = [],
    license = 'MIT',
)



import os

from io import FileIO
from .result import Result
from .result_file import ResultFile
from .upload_io import UploadIO

def build(resource):
    if isinstance(resource, Result):
        return resource.file.url

    if isinstance(resource, ResultFile):
        return resource.url

    if isinstance(resource, UploadIO):
        return resource.file_id

    if isinstance(resource, FileIO):
        return UploadIO(resource).file_id

    if os.path.isfile(resource):
        io = open(resource, 'rb')
        return UploadIO(io).file_id

    return resource



__version__ = '1.5.0'

from .exceptions import *
from .client import Client
from .upload_io import UploadIO
from .api import convert, user

# configuration

api_secret = None
base_uri = 'https://v2.convertapi.com/'
user_agent = 'ConvertAPI-Python/' + __version__
timeout = 1800
conversion_timeout = None
conversion_timeout_delta = 10
upload_timeout = 1800
download_timeout = 1800
max_parallel_uploads = 10

client = Client()



import convertapi

from convertapi import file_param, format_detector, utils
from .result import Result

DEFAULT_URL_FORMAT = 'web'

class Task:
    def __init__(self, from_format, to_format, params, timeout = None):
        self.from_format = from_format
        self.to_format = to_format
        self.params = params
        self.timeout = timeout or convertapi.conversion_timeout

        self.default_params = {
            'Timeout': self.timeout,
            'StoreFile': True,
        }

    def run(self):
        params = self.__normalize_params()
        from_format = self.from_format or self.__detect_format()
        timeout = self.timeout + convertapi.conversion_timeout_delta if self.timeout else None
        path = "convert/%s/to/%s" % (from_format, self.to_format)

        if 'converter' in params:
            path += "/converter/%s" % (params['converter'])

        response = convertapi.client.post(path, params, timeout = timeout)

        return Result(response)

    def __normalize_params(self):
        params = {}

        for k, v in self.params.items():
            if k == 'File':
                params[k] = file_param.build(v)
            elif k == 'Files':
                results = utils.map_in_parallel(file_param.build, v, convertapi.max_parallel_uploads)

                for idx, val in enumerate(results):
                    key = '%s[%i]' % (k, idx)
                    params[key] = val
            else:
                params[k] = v

        params.update(self.default_params)

        return params

    def __detect_format(self):
        if str(self.to_format).lower() == 'zip':
            return 'any'

        if 'Url' in self.params:
            return DEFAULT_URL_FORMAT

        if 'File' in self.params:
            return format_detector.detect(self.params['File'])

        if 'Files' in self.params:
            return format_detector.detect(self.params['Files'][0])
